#ifndef _MRM_GPS_LIBRARY_H
#define _MRM_GPS_LIBRARY_H

#ifndef GPS_API
	#if defined (_WIN32) || defined (WINCE)
		#ifdef DLL_EXPORTS
		#define GPS_API __declspec( dllexport )
		#else
		#define GPS_API __declspec( dllimport )
		#endif
	#elif defined (__linux__) || defined (ANDROID)
		#define GPS_API
		#ifndef _stdcall
			#define _stdcall
		#endif
		#ifndef NULL
			#define NULL 0
		#endif
	#else
		#error:Unknown OS
	#endif
#endif

#include "mrm_error.h"

#ifdef __cplusplus
extern "C" {
#else

typedef unsigned char bool;

#define true 1
#define false 0

#endif

#define GPS_MAXIMUM_LIBRARY_STRING_LENGTH 24

typedef struct
{
	double latitude;
	double longitude;
	double altitude;
} gps_location_t;

typedef struct
{
	unsigned short year;		
	unsigned char month;
	unsigned char day;
	unsigned char hour;
	unsigned char minute;
	unsigned char second;
	unsigned short msecond;
} gps_time_t;

typedef struct
{
    int prn; /* satellite ID */
    int elevation; /* Elevation in Degrees */
    int azimuth; /* Azimuth degrees from true North */
    int snr; /* Satellite SNR in dB */
    int used; /* 1 = used , otherwise not */
} gps_satellite_t;

typedef enum gps_fix_status
{
	GPS_STATUS_NO_FIX = 0,
	GPS_STATUS_GPS,
	GPS_STATUS_DGPS,
	GPS_STATUS_DR
}gps_fix_status;

typedef enum gps_antenna_status
{
	GPS_ANTENNA_UNKNOWN = 0,
	GPS_ANTENNA_OK,
	GPS_ANTENNA_OPEN,
	GPS_ANTENNA_SHORT	
}gps_antenna_status;

typedef void ( _stdcall *gps_nmea_callback ) ( char *NMEA , int length);

GPS_API mrm_err gps_init(char *port);
GPS_API mrm_err gps_deinit(void);
GPS_API mrm_err gps_get_version(char *version);
GPS_API mrm_err gps_get_location(gps_location_t *location);
GPS_API mrm_err gps_get_speed(double *mps);
GPS_API mrm_err gps_get_utc_time(gps_time_t *utc_time);
GPS_API mrm_err gps_get_satellite_in_view(int *count);
GPS_API mrm_err gps_get_satellites(gps_satellite_t *satellite);
GPS_API mrm_err gps_get_fix_status(gps_fix_status *status);
GPS_API mrm_err gps_get_antenna_status(gps_antenna_status *status);
GPS_API mrm_err gps_set_nmea_callback(gps_nmea_callback callback);
GPS_API mrm_err gps_clear_nmea_callback(void);

#ifdef __cplusplus
}
#endif

#endif
