#ifndef _MRM_IVCP_PERIPHERAL_CONTROL_H
#define _MRM_IVCP_PERIPHERAL_CONTROL_H

#include "ivcp/library.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum ivcp_peripheral_control_type
{
   IVCP_PERIPHERAL_WWAN_POWER = 0,
   IVCP_PERIPHERAL_WIFI_POWER,
   IVCP_PERIPHERAL_GPS_POWER,
   IVCP_PERIPHERAL_SIM_SWITCH,
   IVCP_PERIPHERAL_REARVIEW_SWITCH,
   IVCP_PERIPHERAL_MIC_SWITCH
}ivcp_peripheral_control_type;

typedef enum ivcp_peripheral_power_id
{
   IVCP_PERIPHERAL_PID_WWAN = 0,
   IVCP_PERIPHERAL_PID_WIFI,
   IVCP_PERIPHERAL_PID_GPS
}ivcp_peripheral_power_id;

typedef enum ivcp_peripheral_sim_id
{
   IVCP_PERIPHERAL_SID_SIM0 = 0,
   IVCP_PERIPHERAL_SID_SIM1
}ivcp_peripheral_sim_id;

typedef enum ivcp_peripheral_rearview_id
{
   IVCP_PERIPHERAL_RID_MAIN = 0,
   IVCP_PERIPHERAL_RID_EXTERNEL1,
   IVCP_PERIPHERAL_RID_EXTERNEL2,
   IVCP_PERIPHERAL_RID_EXTERNEL3,
}ivcp_peripheral_rearview_id;

typedef enum ivcp_peripheral_comport_modes
{
	IVCP_PERIPHERAL_COMPORT_MODE_RS232 = 0,
	IVCP_PERIPHERAL_COMPORT_MODE_RS485,
	IVCP_PERIPHERAL_COMPORT_MODE_RS422
}ivcp_peripheral_comport_modes;

typedef enum ivcp_peripheral_comport
{
	IVCP_PERIPHERAL_COMPORT_A = 0,
	IVCP_PERIPHERAL_COMPORT_B,
	IVCP_PERIPHERAL_COMPORT_C,
	IVCP_PERIPHERAL_COMPORT_D,
	IVCP_PERIPHERAL_COMPORT_E,
	IVCP_PERIPHERAL_COMPORT_F
}ivcp_peripheral_comport;

typedef enum ivcp_peripheral_voice_input
{
   IVCP_PERIPHERAL_VOICE_INPUT_FRONT_MIC = 0,
   IVCP_PERIPHERAL_VOICE_INPUT_MIC_IN,
   IVCP_PERIPHERAL_VOICE_BLUETOOTH
}ivcp_peripheral_voice_input;

typedef struct
{
	ivcp_peripheral_comport_modes comport_mode;
	char term_status;
} ivcp_peripheral_comport_mode_t;

IVCP_API mrm_err ivcp_peripheral_control_available(ivcp_peripheral_control_type control_type, char *supported);
IVCP_API mrm_err ivcp_peripheral_power_on(ivcp_peripheral_power_id id);
IVCP_API mrm_err ivcp_peripheral_power_off(ivcp_peripheral_power_id id);
IVCP_API mrm_err ivcp_peripheral_get_power_status(ivcp_peripheral_power_id id, char *status);
IVCP_API mrm_err ivcp_peripheral_wwan_wakeup_enable();
IVCP_API mrm_err ivcp_peripheral_wwan_wakeup_disable();
IVCP_API mrm_err ivcp_peripheral_get_wwan_wakeup_status(char *status);
IVCP_API mrm_err ivcp_peripheral_get_sim(ivcp_peripheral_sim_id *id);
IVCP_API mrm_err ivcp_peripheral_set_sim(ivcp_peripheral_sim_id id);
IVCP_API mrm_err ivcp_peripheral_get_rearview(ivcp_peripheral_rearview_id *id);
IVCP_API mrm_err ivcp_peripheral_set_rearview(ivcp_peripheral_rearview_id id);
IVCP_API mrm_err ivcp_peripheral_auto_rearview_enable();
IVCP_API mrm_err ivcp_peripheral_auto_rearview_disable();
IVCP_API mrm_err ivcp_peripheral_get_auto_rearview_status(char *status);
IVCP_API mrm_err ivcp_peripheral_get_comport_mode(ivcp_peripheral_comport_modes *mode, bool *termination_status);
IVCP_API mrm_err ivcp_peripheral_set_comport_mode(ivcp_peripheral_comport_modes mode, bool termination_status);
IVCP_API mrm_err ivcp_peripheral_get_voice_input(ivcp_peripheral_voice_input *type);
IVCP_API mrm_err ivcp_peripheral_set_voice_input(ivcp_peripheral_voice_input type);
IVCP_API mrm_err ivcp_peripheral_get_gps_antenna_status(char *status);

#ifdef __cplusplus
}
#endif
#endif
