#ifndef _MRM_IVCP_POWER_MANAGEMENT_H
#define _MRM_IVCP_POWER_MANAGEMENT_H

#include "ivcp/library.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum ivcp_ignition_mode
{
   IVCP_IGNITION_MODE_POWER_OFF = 0,
   IVCP_IGNITION_MODE_SUSPEND
}ivcp_ignition_mode;

typedef enum ivcp_power_mode
{
	IVCP_POWER_MODE_48V = 1,
	IVCP_POWER_MODE_24V = 2,
	IVCP_POWER_MODE_12V = 3
}ivcp_power_mode;

typedef enum ivcp_pm_event
{
   IVCP_EVENT_LOW_VOLTAGE = 0,
   IVCP_EVENT_LOW_VOLTAGE_HARD = 1,
   IVCP_EVENT_IGNITION_ON = 2,
   IVCP_EVENT_IGNITION_OFF_TO_POWER_OFF = 3,
   IVCP_EVENT_IGNITION_OFF_HARD = 4,
   IVCP_EVENT_POST_BOOT_POWER_CHECK = 5,
   IVCP_EVENT_IGNITION_OFF_TO_SUSPEND = 6,
   IVCP_EVENT_POWER_OFF_ALARM = 7
}ivcp_pm_event;

typedef enum ivcp_pm_wakeup_type
{
	IVCP_WAKEUP_TYPE_POWER_BUTTON = 0,
	IVCP_WAKEUP_TYPE_IGNITION,
	IVCP_WAKEUP_TYPE_WWAN,
	IVCP_WAKEUP_TYPE_GSENSOR,
	IVCP_WAKEUP_TYPE_DI1,
	IVCP_WAKEUP_TYPE_DI2,
	IVCP_WAKEUP_TYPE_ALARM,
	IVCP_WAKEUP_TYPE_HOTKEY,
	IVCP_WAKEUP_TYPE_DI3,
	IVCP_WAKEUP_TYPE_DI4,
	IVCP_WAKEUP_TYPE_KEEP_ALIVE_MODE,
	IVCP_WAKEUP_TYPE_AT_MODE,
	IVCP_WAKEUP_TYPE_RESET,
	IVCP_WAKEUP_TYPE_DI_UPDATE
}ivcp_pm_wakeup_type;

typedef enum ivcp_pm_shutdown_mask
{
	IVCP_SHUTDOWN_MASK_POWER_BUTTON = 0,
	IVCP_SHUTDOWN_MASK_IGNITION
}ivcp_pm_shutdown_mask;


IVCP_API mrm_err ivcp_pm_power_off(void);
IVCP_API mrm_err ivcp_pm_get_ignition_status(char *status);
IVCP_API mrm_err ivcp_pm_ignition_wakeup_enable(void);
IVCP_API mrm_err ivcp_pm_ignition_wakeup_disable(void);
IVCP_API mrm_err ivcp_pm_get_ignition_wakeup_status(char *status);
IVCP_API mrm_err ivcp_pm_set_ignition_poweroff_event(void *poweroff_event);

IVCP_API mrm_err ivcp_pm_get_shutdown_mask(ivcp_pm_shutdown_mask mask, char *status);
IVCP_API mrm_err ivcp_pm_set_shutdown_mask(ivcp_pm_shutdown_mask mask, char status);

IVCP_API mrm_err ivcp_pm_power_off_alarm_enable();
IVCP_API mrm_err ivcp_pm_power_off_alarm_disable();
IVCP_API mrm_err ivcp_pm_get_power_off_alarm_status(char *status);

IVCP_API mrm_err ivcp_pm_get_voltage(float *voltage);
IVCP_API mrm_err ivcp_pm_get_alive_mode(char *mode);
IVCP_API mrm_err ivcp_pm_set_alive_mode(char mode);
IVCP_API mrm_err ivcp_pm_get_at_mode(char *mode);
IVCP_API mrm_err ivcp_pm_set_at_mode(char mode);
IVCP_API mrm_err ivcp_pm_get_event_delay(ivcp_pm_event event_type, unsigned int *second);
IVCP_API mrm_err ivcp_pm_set_event_delay(ivcp_pm_event event_type, unsigned int second);
IVCP_API mrm_err ivcp_pm_get_last_wakeup_source(ivcp_pm_wakeup_type *source);
IVCP_API mrm_err ivcp_pm_set_power_mode(char mode);
IVCP_API mrm_err ivcp_pm_get_power_mode(char *mode);
IVCP_API mrm_err ivcp_pm_get_power_status(char *status);

IVCP_API mrm_err ivcp_pm_get_lvp_range(float *min, float *max, float *defualt);
IVCP_API mrm_err ivcp_pm_get_lvp_preboot_threshold(float *voltage);
IVCP_API mrm_err ivcp_pm_set_lvp_preboot_threshold(float voltage);
IVCP_API mrm_err ivcp_pm_get_lvp_postboot_threshold(float *voltage);
IVCP_API mrm_err ivcp_pm_set_lvp_postboot_threshold(float voltage);
IVCP_API mrm_err ivcp_pm_reset_lvp_threshold(void);
IVCP_API mrm_err ivcp_pm_lvp_preboot_enable(void);
IVCP_API mrm_err ivcp_pm_lvp_preboot_disable(void);
IVCP_API mrm_err ivcp_pm_lvp_postboot_enable(void);
IVCP_API mrm_err ivcp_pm_lvp_postboot_disable(void);
IVCP_API mrm_err ivcp_pm_lvp_preboot_get_status(char *status);
IVCP_API mrm_err ivcp_pm_lvp_postboot_get_status(char *status);
IVCP_API mrm_err ivcp_pm_set_lvp_poweroff_event(void *poweroff_event);

IVCP_API mrm_err ivcp_pm_force_shutdown_enable();
IVCP_API mrm_err ivcp_pm_force_shutdown_disable();
IVCP_API mrm_err ivcp_pm_get_force_shutdown_status(char* status);
IVCP_API mrm_err ivcp_pm_get_force_shutdown_delay(unsigned int* second);
IVCP_API mrm_err ivcp_pm_set_force_shutdown_delay(unsigned int second);


#ifdef __cplusplus
}
#endif
#endif
