#ifndef _MRM_VCIL_CAN_H
#define _MRM_VCIL_CAN_H

#include "vcil/library.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum vcil_can_speed
{
	VCIL_CAN_SPEED_125K = 0,
    VCIL_CAN_SPEED_250K = 1,
    VCIL_CAN_SPEED_500K = 2,
    VCIL_CAN_SPEED_1M   = 3,
	VCIL_CAN_SPEED_200K = 4,
	VCIL_CAN_SPEED_100K = 5,
	VCIL_CAN_SPEED_800K = 6,
	VCIL_CAN_SPEED_83K = 7,
	VCIL_CAN_SPEED_50K = 8,
	VCIL_CAN_SPEED_20K = 9,
	VCIL_CAN_SPEED_10K = 10,
	VCIL_CAN_SPEED_5K = 11,
	VCIL_CAN_SPEED_BUS_OFF = 0xFE,
	VCIL_CAN_SPEED_USER_DEFINE = 0xFF
} vcil_can_speed;

typedef enum vcil_can_bus_mode
{
	VCIL_CAN_BUS_NORMAL_MODE = 0,
    VCIL_CAN_BUS_LISTEN_MODE = 1,
    VCIL_CAN_BUS_INIT_MODE = 2
} vcil_can_bus_mode;

typedef struct
{
	unsigned char port;
	char length;
	bool remote_request;
	bool extended_frame;
	unsigned int id;
	unsigned char data[8];
} vcil_can_message_t;

typedef struct
{
	unsigned char type;
	unsigned char bank;
	bool remote_request;
	bool extended_frame;
	unsigned int id1;
	unsigned int mask1;
	unsigned int id2;
	unsigned int mask2;
} vcil_can_mask_t;

typedef struct
{
	unsigned int can_port0_recv_count;
	unsigned int can_port0_send_count;
	unsigned int can_port0_overrun_count;
	unsigned int can_port1_recv_count;
	unsigned int can_port1_send_count;
	unsigned int can_port1_overrun_count;
} vcil_can_stat_t;

typedef struct
{
	unsigned int rec; /* receive error counter */
	unsigned int tec; /* transmit error counter */
	unsigned int last_error_code; /* last error code */
	unsigned int error_flag;
} vcil_can_error_status;

VCIL_API mrm_err vcil_can_read(vcil_can_message_t *message);
VCIL_API mrm_err vcil_can_write(vcil_can_message_t *message);
VCIL_API mrm_err vcil_can_set_speed(unsigned char port, vcil_can_speed speed);
VCIL_API mrm_err vcil_can_set_speed_listen_mode(unsigned char port, vcil_can_speed speed);
VCIL_API mrm_err vcil_can_get_speed(unsigned char port, vcil_can_speed *speed, vcil_can_bus_mode *mode);
VCIL_API mrm_err vcil_can_get_bus_error_status(unsigned char port, vcil_can_error_status *status);
VCIL_API mrm_err vcil_can_set_mask(unsigned char port, vcil_can_mask_t *mask);
VCIL_API mrm_err vcil_can_get_mask(unsigned char port, vcil_can_mask_t *mask);
VCIL_API mrm_err vcil_can_remove_mask(unsigned char port, unsigned char bank);
VCIL_API mrm_err vcil_can_reset_mask(unsigned char port);
VCIL_API mrm_err vcil_can_set_event(void *can_rx_event);
VCIL_API mrm_err vcil_can_get_stat(vcil_can_stat_t *stat);

#ifdef __cplusplus
}
#endif

#endif
