#ifndef _MRM_VCIL_J1939_H
#define _MRM_VCIL_J1939_H

#include "vcil/library.h"

#ifdef __cplusplus
extern "C" {
#endif

#define	VCIL_MAX_J1939_DATA_SIZE 64
#define	VCIL_MAX_J1939_MASK_NUM 128

typedef struct
{
	unsigned char port;
	unsigned int pgn; /* parameter group number. */
	unsigned char destination;
	unsigned char source;
	unsigned char priority;
	int length;
	unsigned char data[VCIL_MAX_J1939_DATA_SIZE];
} vcil_j1939_message_t;

typedef struct
{
    unsigned char address;
	unsigned char arbitrary_address_capable;
	unsigned char industry_group;
	unsigned char vehicle_system_instance;
	unsigned char vehicle_system;
	unsigned char function;
	unsigned char function_instance;
	unsigned char ecu_instance;
	unsigned short manufacturer_code;
	unsigned int identity_number;
	unsigned char address_claim_on_starup;
} vcil_j1939_config_t;

typedef struct
{
	unsigned int j1939_port0_recv_count;
	unsigned int j1939_port0_send_count;
	unsigned int j1939_port0_fail_count;
	unsigned int j1939_port1_recv_count;
	unsigned int j1939_port1_send_count;
	unsigned int j1939_port1_fail_count;
} vcil_j1939_stat_t;

VCIL_API mrm_err vcil_j1939_read(vcil_j1939_message_t *message);
VCIL_API mrm_err vcil_j1939_write(vcil_j1939_message_t *message);
VCIL_API mrm_err vcil_j1939_add_mask(unsigned char port, unsigned int pgn);
VCIL_API mrm_err vcil_j1939_get_mask_number(unsigned char port,  unsigned int* total);
VCIL_API mrm_err vcil_j1939_get_all_mask(unsigned char port, unsigned int* pgn);
VCIL_API mrm_err vcil_j1939_remove_mask(unsigned char port, unsigned int pgn);
VCIL_API mrm_err vcil_j1939_remove_all_mask(unsigned char port);
VCIL_API mrm_err vcil_j1939_set_event(void *j1939_rx_event);
VCIL_API mrm_err vcil_j1939_set_config(unsigned char port, vcil_j1939_config_t *config);
VCIL_API mrm_err vcil_j1939_get_config(unsigned char port, vcil_j1939_config_t *config);
VCIL_API mrm_err vcil_j1939_get_stat(vcil_j1939_stat_t *stat);

#ifdef __cplusplus
}
#endif

#endif