#include <stdio.h>

#include "common.h"
#include "function.h"
#include "ivcp/ivcp.h"

char library_version[IVCP_MAXIMUM_LIBRARY_STRING_LENGTH]={0};
char platform_name[IVCP_MAXIMUM_PLATFORM_STRING_LENGTH]={0};

static const char *page_name[] = {
	"Firmware management",
	"Watchdog control",
	"Alarm Control",
	"Peripheral control",
	"Digital IO Control",
	"Storage",
	"PSensor",
	"GSensor",
	"Power Management",
	"System Sensor"
};

static const int page_function_supported[] = {
	IVCP_SAMPLE_CODE_FUNCTION_FIRMWARE,
	IVCP_SAMPLE_CODE_FUNCTION_WATCHDOG,
	IVCP_SAMPLE_CODE_FUNCTION_ALARM,
	IVCP_SAMPLE_CODE_FUNCTION_PERIPHERAL_CONTROL,
	IVCP_SAMPLE_CODE_FUNCTION_DIGITAL_IO,
	IVCP_SAMPLE_CODE_FUNCTION_STORAGE,
	IVCP_SAMPLE_CODE_FUNCTION_PSENSOR,
	IVCP_SAMPLE_CODE_FUNCTION_GSENSOR,
	IVCP_SAMPLE_CODE_FUNCTION_POWER_MANAGEMENT,
	IVCP_SAMPLE_CODE_FUNCTION_SENSOR,
	IVCP_SAMPLE_CODE_MAX_FUNCTION_PAGE
};

static void show_welcome_title(void)
{
	printf("IVCP SDK Sample -- library version:%s\n", library_version);
	printf("            -- platform:%s\n", platform_name);
	printf(">> Main Menu\n\n");
}

static void show_menu(void)
{
	printf("0) exit\n");
	
	for (int i = 0; i < IVCP_SAMPLE_CODE_MAX_FUNCTION_PAGE - 1; i++)
	{
		if (page_function_supported[i] >= 0)
			printf("%d) %s\n", i + 1, page_name[i]);
		else
			break;
	}
	
	printf("\nEnter your choice: ");
}

int main(int argc, char *argv[])
{
	mrm_err result;
	int op;

	if( (result = ivcp_init()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_init fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}
	
	if( (result = ivcp_get_version(library_version)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_get_version fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}
	
	if( (result = ivcp_get_platform_name(platform_name)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_get_platform_name fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	} 

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		show_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;

		switch(op)
		{
			case IVCP_SAMPLE_CODE_FUNCTION_FIRMWARE:
				firmware_main();
				break;
			case IVCP_SAMPLE_CODE_FUNCTION_WATCHDOG:
				watchdog_main();
				break;
			case IVCP_SAMPLE_CODE_FUNCTION_ALARM:
				alarm_main();
				break;
			case IVCP_SAMPLE_CODE_FUNCTION_PERIPHERAL_CONTROL:
				peripheral_main();
				break;
			case IVCP_SAMPLE_CODE_FUNCTION_DIGITAL_IO:
				dio_main();
				break;
			case IVCP_SAMPLE_CODE_FUNCTION_STORAGE:
				storage_main();
				break;
			case IVCP_SAMPLE_CODE_FUNCTION_PSENSOR:
				psensor_main();
				break;
			case IVCP_SAMPLE_CODE_FUNCTION_GSENSOR:
				gsensor_main();
				break;
			case IVCP_SAMPLE_CODE_FUNCTION_POWER_MANAGEMENT:
				power_management_main();
				break;
			case IVCP_SAMPLE_CODE_FUNCTION_SENSOR:
				sensor_main();
				break;
			default:
				printf("Unknown choice!\n");
				printf("\nPress ENTER to continue. ");
				WAIT_ENTER();
				break;
		}
	}

	if( (result = ivcp_deinit()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_deinit. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}

	return 0;
}


