#include <stdio.h>

#include "common.h"
#include "ivcp/library.h"
#include "ivcp/alarm.h"

/*

IVCP_API mrm_err ivcp_alarm_wakeup_enable(void);
IVCP_API mrm_err ivcp_alarm_wakeup_disable(void);
IVCP_API mrm_err ivcp_alarm_get_wakeup_status(char *status);
*/

enum page_function{
	ALARM_GET_REAL_TIME = 1,
	ALARM_SET_REAL_TIME,
	ALARM_GET_WAKEUP_TIME,
	ALARM_SET_WAKEUP_TIME,
	ALARM_GET_WAKEUP_MODE,
	ALARM_SET_WAKEUP_MODE,
	ALARM_ENABLE_WAKEUP,
	ALARM_DISABLE_WAKEUP,
	ALARM_GET_WAKEUP_STATUS
};

static void show_welcome_title(void)
{
	printf("IVCP SDK Sample -- library version:%s\n", library_version);
	printf(">> Alarm Control Menu\n\n");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) Alarm Get Real Time\n", ALARM_GET_REAL_TIME);
	printf("%d) Alarm Set Real Time\n", ALARM_SET_REAL_TIME);
	printf("%d) Alarm Get Wakeup Time\n", ALARM_GET_WAKEUP_TIME);
	printf("%d) Alarm Set Wakeup Time\n", ALARM_SET_WAKEUP_TIME);
	printf("%d) Alarm Get Wakeup Mode\n", ALARM_GET_WAKEUP_MODE);
	printf("%d) Alarm Set Wakeup Mode\n", ALARM_SET_WAKEUP_MODE);
	printf("%d) Alarm Wakeup Enable\n", ALARM_ENABLE_WAKEUP);
	printf("%d) Alarm Wakeup Disable\n", ALARM_DISABLE_WAKEUP);
	printf("%d) Alarm Get Wakeup Status\n", ALARM_GET_WAKEUP_STATUS);
		
	printf("\nEnter your choice: ");
}

void alarm_get_realtime()
{
	mrm_err result;
	
	ivcp_alarm_real_time_t real_time;
	
	if( (result = ivcp_alarm_get_real_time(&real_time)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_alarm_get_real_time fail. error code=0x%08x\n", result);
		return;
	}
	
	int YY=real_time.year, MM=real_time.month, DD=real_time.day, HH=real_time.hour, mm=real_time.minute, SS=real_time.second, WW=real_time.week;
	
	printf("The current time is %d/%d/%d - %d:%d:%d Week:%d (0=sunday)\n",YY,MM,DD, HH,mm,SS, WW);
}

void alarm_set_realtime()
{
	printf("Real time Year (2000-2099):");
	unsigned short YY;
	scanf("%hu", &YY);
	
	WAIT_ENTER();
	
	if( YY < 2000 || YY > 2099 )
	{
		printf("Error range\n");
		return;
	}
	
	printf("Real time Month (1-12):");
	unsigned short MM;
	scanf("%hu", &MM);
	
	WAIT_ENTER();
	
	if( MM < 1 || MM > 12 )
	{
		printf("Error range\n");
		return;
	}
	
	printf("Real time Day (1-31):");
	unsigned short DD;
	scanf("%hu", &DD);
	
	WAIT_ENTER();
		
	switch(MM)
	{
		case 1:
		case 3:
		case 5:
		case 7:
		case 8:
		case 10:
		case 12:
			{
				if( DD < 1 || DD > 31 )
				{
					printf("Error range\n");
					return;
				}
			}
			break;
		case 4:
		case 6:
		case 9:
		case 11:
			{
				if( DD < 1 || DD > 30 )
				{
					printf("Error range\n");
					return;
				}
			}		
		case 2:
			{
				if( DD < 1 || DD > 29 )
				{
					printf("Error range\n");
					return;
				}
				
				if( YY%4 !=0 && DD == 29)
				{
					printf("Error range\n");
					return;
				}
			}
			break;
		default:
			printf("Error range\n");
			return;	
	}
	
	printf("Real time Hour(0-23):");
	unsigned short HH;
	scanf("%hu", &HH);
	
	WAIT_ENTER();
	
	if( HH > 23 )
	{
		printf("Error range\n");
		return;
	}
	
	printf("Real time Minute(0-59):");
	unsigned short mm;
	scanf("%hu", &mm);
	
	WAIT_ENTER();
	
	if( mm > 59 )
	{
		printf("Error range\n");
		return;
	}
	
	printf("Real time second(0-59):");
	unsigned short SS;
	scanf("%hu", &SS);
	
	WAIT_ENTER();
	
	if( SS > 59 )
	{
		printf("Error range\n");
		return;
	}
	
	printf("Real time week(0-6, 0=sunday) :");
	unsigned short WW;
	scanf("%hu", &WW);
	
	WAIT_ENTER();
	
	if( WW > 6 )
	{
		printf("Error range\n");
		return;
	}
	
	ivcp_alarm_real_time_t real_time;
		
	mrm_err result;
	
	real_time.year = YY;		
	real_time.month = MM;
	real_time.day = DD;
	real_time.hour = HH;
	real_time.minute= mm;
	real_time.second = SS;
	real_time.week = WW;

	if( (result = ivcp_alarm_set_real_time(real_time)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_alarm_set_real_time fail. error code=0x%08x\n", result);
		return;
	}


	printf("Success\n");
}

void alarm_get_wakeup_time()
{
	mrm_err result;
		
	ivcp_alarm_wakeup_time_t wakeup_time;
		
	if( (result = ivcp_alarm_get_wakeup_time(&wakeup_time)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_alarm_get_wakeup_time fail. error code=0x%08x\n", result);
		return;
	}
	
	int DoW=wakeup_time.day_of_week, HH=wakeup_time.hour, mm=wakeup_time.minute;
	
	printf("The wakeup time is Day of week= %d(0=sunday), Hour=%d, Minute=%d\n",DoW, HH, mm);
}

void alarm_set_wakeup_time()
{
	printf("Wakeup time Day of week (0-6, 0=sunday) :");
	unsigned short WW;
	scanf("%hu", &WW);
	
	WAIT_ENTER();
	
	if( WW > 6 )
	{
		printf("Error range\n");
		return;
	}
	
	printf("Wakeup time Hour(0-23):");
	unsigned short HH;
	scanf("%hu", &HH);
	
	WAIT_ENTER();
	
	if( HH > 23 )
	{
		printf("Error range\n");
		return;
	}
	
	printf("Wakeup time Minute(0-59):");
	unsigned short mm;
	scanf("%hu", &mm);
	
	WAIT_ENTER();
	
	if( mm > 59 )
	{
		printf("Error range\n");
		return;
	}

	ivcp_alarm_wakeup_time_t wakeup_time;
		
	mrm_err result;
	
	wakeup_time.day_of_week = WW;
	wakeup_time.hour = HH;
	wakeup_time.minute= mm;

	if( (result = ivcp_alarm_set_wakeup_time(wakeup_time)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_alarm_set_wakeup_time fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void alarm_get_wakeup_mode()
{
	mrm_err result;
	
	ivcp_alarm_mode mode;
	
	if( (result = ivcp_alarm_get_wakeup_mode(&mode)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_alarm_get_wakeup_mode fail. error code=0x%08x\n", result);
		return;
	}
	
	if( mode == IVCP_ALARM_MODE_NO_ALARM )
		printf("The Alarm Wakeup Disable\n");
	else if( mode == IVCP_ALARM_MODE_HOURLY )
		printf("The Alarm Wakeup Enable by hourly\n");
	else if( mode == IVCP_ALARM_MODE_DAILY )
		printf("The Alarm Wakeup Enable by Daily\n");
	else if( mode == IVCP_ALARM_MODE_WEEKLY )
		printf("The Alarm Wakeup Enable by Weekly\n");
}

void alarm_set_wakeup_mode()
{
	printf("Alarm Wakeup Mode:\n");
	printf("0) Horly:\n");
	printf("1) Daily:\n");
	printf("2) Weekly:\n");
	
	unsigned short m;
	scanf("%hu", &m);
	
	WAIT_ENTER();
	
	if( m > 2 )
	{
		printf("Error range\n");
		return;
	}

	ivcp_alarm_mode mode=(ivcp_alarm_mode)m;
		
	mrm_err result;	

	if( (result = ivcp_alarm_set_wakeup_mode(mode)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_alarm_set_wakeup_mode fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void alarm_wakeup_disable()
{
	mrm_err result;
	
	if( (result = ivcp_alarm_wakeup_disable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_alarm_wakeup_disable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void alarm_wakeup_enable()
{
	mrm_err result;
	
	if( (result = ivcp_alarm_wakeup_enable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_alarm_wakeup_enable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void alarm_wakeup_get_status()
{
	mrm_err result;
	
	char status;
	
	if( (result = ivcp_alarm_get_wakeup_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_alarm_get_wakeup_status fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Alarm Wakeup is %s.\n", status ? "Enable" : "Disable" );
}

void alarm_main()
{
	int op;
	
	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
	
		switch(op)
		{
			case ALARM_GET_REAL_TIME:
				alarm_get_realtime();
				break;
			case ALARM_SET_REAL_TIME:
				alarm_set_realtime();
				break;
			case ALARM_GET_WAKEUP_TIME:
				alarm_get_wakeup_time();
				break;
			case ALARM_SET_WAKEUP_TIME:
				alarm_set_wakeup_time();
				break;
			case ALARM_GET_WAKEUP_MODE:
				alarm_get_wakeup_mode();
				break;
			case ALARM_SET_WAKEUP_MODE:
				alarm_set_wakeup_mode();
				break;
			case ALARM_ENABLE_WAKEUP:
				alarm_wakeup_enable();
				break;
			case ALARM_DISABLE_WAKEUP:
				alarm_wakeup_disable();
				break;
			case ALARM_GET_WAKEUP_STATUS:
				alarm_wakeup_get_status();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
