#include <stdio.h>

#include "common.h"
#include "ivcp/library.h"
#include "ivcp/digital_io.h"

int di_number = 0;
int do_number = 0;

enum page_function{
	DIO_READ_INPUT = 1,
	DIO_WRITE_OUTPUT,
	DIO_READ_OUTPUT
};

static void show_welcome_title(void)
{
	printf("IVCP SDK Sample -- library version:%s\n", library_version);
	printf(">> Digital IO Control Menu\n\n");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) Read DI Status\n", DIO_READ_INPUT);
	printf("%d) Write DO Status\n", DIO_WRITE_OUTPUT);
	printf("%d) Read DO Status\n", DIO_READ_OUTPUT);
		
	printf("\nEnter your choice: ");
}

int dio_init()
{
	static int init_once = 0;
	
	if( init_once )
		return 0;
	
	mrm_err result;
	
	char data;
	
	if( (result = ivcp_dio_get_input_number(&data)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_dio_get_input_number fail. error code=0x%08x\n", result);
		return -1;
	}
	di_number = data;
	
	if( (result = ivcp_dio_get_output_number(&data)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_dio_get_output_number fail. error code=0x%08x\n", result);
		return -1;
	}
	do_number = data;
	
	init_once = 1;
	
	return 0;
}

void read_digital_input()
{
	mrm_err result;
	
	char status;
	for(int i = 0; i < di_number ; i++)
	{
		if( (result = ivcp_dio_read_input( (ivcp_dio_pin_id)i, &status)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_dio_read_input %d fail. error code=0x%08x\n", i, result);
			return;
		}
		
		printf("PIN-%d = %s\n", i, status ? "ON" : "OFF" );
	}
}

void write_digital_output()
{
	printf("Pin (0-%d):", di_number-1);
	int p;
	scanf("%d", &p);
	
	WAIT_ENTER();
	
	if( p < 0 || p >=di_number)
	{
		printf("Error range\n");
		return;
	}
	
	printf("On/Off (0-1):");
	int s;
	scanf("%d", &s);
	
	WAIT_ENTER();
	
	if( s < 0 || s >1)
	{
		printf("Error range\n");
		return;
	}
		
	mrm_err result;
	
	char status = s;

	if( (result = ivcp_dio_write_output((ivcp_dio_pin_id)p, status)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_dio_write_output %d fail. error code=0x%08x\n", p, result);
		return;
	}


	printf("Success\n");
}

void read_digital_output()
{
	mrm_err result;
	
	char status;
	for(int i = 0; i < do_number ; i++)
	{
		if( (result = ivcp_dio_read_output((ivcp_dio_pin_id)i, &status)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_dio_read_output %d fail. error code=0x%08x\n", i, result);
			return;
		}
		
		printf("PIN%d = %s\n", i, status ? "ON" : "OFF" );
	}
}

void dio_main()
{
	int op;
	
	if( dio_init() < 0 )
		return ;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;

		switch(op)
		{
			case DIO_READ_INPUT:
				read_digital_input();
				break;
			case DIO_WRITE_OUTPUT:
				write_digital_output();
				break;
			case DIO_READ_OUTPUT:
				read_digital_output();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
