#include <stdio.h>

#include "common.h"
#include "ivcp/library.h"
#include "ivcp/gsensor.h"

enum page_function{
	GSENSOR_ENABLE = 1,
	GSENSOR_DISABLE,
	GET_GSENSOR_STATUS,
	GSENSOR_READ,
	GET_GSENSOR_RESOLUTION,
	SET_GSENSOR_RESOLUTION,
	GSENSOR_WAKEUP_ENABLE,
	GSENSOR_WAKEUP_DISABLE,
	GET_GSENSOR_WAKEUP_STATUS,
	GET_GSENSOR_WAKEUP_THRESHOLD,
	SET_GSENSOR_WAKEUP_THRESHOLD
};

static void show_welcome_title(void)
{
	printf("IVCP SDK Sample -- library version:%s\n", library_version);
	printf(">> Gsensor Control Menu\n\n");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) Enable Gsensor\n", GSENSOR_ENABLE);
	printf("%d) Disable Gsensor\n", GSENSOR_DISABLE);
	printf("%d) Get Gsensor Status\n", GET_GSENSOR_STATUS);
	printf("%d) Read Gsensor Value\n", GSENSOR_READ);
	printf("%d) Get Gsensor Resolution\n", GET_GSENSOR_RESOLUTION);
	printf("%d) Set Gsensor Resolution\n",SET_GSENSOR_RESOLUTION);
	printf("%d) Enable Gsensor Wakeup\n", GSENSOR_WAKEUP_ENABLE);
	printf("%d) Disable Gsensor  Wakeup\n", GSENSOR_WAKEUP_DISABLE);
	printf("%d) Get Gsensor Wakeup Status\n", GET_GSENSOR_WAKEUP_STATUS);
	printf("%d) Get Gsensor Wakeup Threshold\n", GET_GSENSOR_WAKEUP_THRESHOLD);
	printf("%d) Set Gsensor Wakeup Threshold\n",SET_GSENSOR_WAKEUP_THRESHOLD);
		
	printf("\nEnter your choice: ");
}

int gsensor_init()
{
	static int init_once = 0;
	
	if( init_once )
		return 0;
	
	mrm_err result;
	
	char supported;
	
	if( (result = ivcp_gsensor_available(&supported)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_available fail. error code=0x%08x\n", result);
		return -1;
	}
	
	if( !supported )
	{
		printf("Gsensor not supported\n");
		WAIT_ENTER();
	}
	
	init_once = 1;
	
	return 0;
}

void gsensor_enable()
{
	mrm_err result;
	
	if( (result = ivcp_gsensor_enable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_enable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void gsensor_disable()
{
	mrm_err result;
	
	if( (result = ivcp_gsensor_disable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_disable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void get_gsensor_status()
{
	mrm_err result;
	
	char status;
	
	if( (result = ivcp_gsensor_get_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_get_status fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Gsensor is %s\n", status ? "Enable" : "Disable" );
}

void read_gsensor()
{
	mrm_err result;
	
	ivcp_gsensor_value_t value;
	
	if( (result = ivcp_gsensor_read( &value )) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_read fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Gsensor Value is x=%d, y=%d,z=%d mg\n", value.x_mg, value.y_mg, value.z_mg);
}

void gsensor_get_resolution()
{
	mrm_err result;
	
	ivcp_gsensor_res res;
	const char *res_string[] = { "2"," 4", "8", "16",};
	
	if( (result = ivcp_gsensor_get_resolution(&res)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_get_resolution fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Current Gsensor Resolution is %sG\n", res_string[(int)res]);
}

void gsensor_set_resolution()
{
	printf("Resolution :\n");
	printf("0) 2G\n");
	printf("1) 4G\n");
	printf("2) 8G\n");
	printf("3) 16G\n");
	
	int r;
	scanf("%d", &r);
	
	WAIT_ENTER();
	
	if( r <0 || r>3 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;

	if( (result = ivcp_gsensor_set_resolution((ivcp_gsensor_res)r)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_set_resolution fail. error code=0x%08x\n", result);
		return;
	}


	printf("Success\n");
}

void gsensor_wakeup_enable()
{
	mrm_err result;
	
	if( (result = ivcp_gsensor_wakeup_enable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_wakeup_enable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void gsensor_wakeup_disable()
{
	mrm_err result;
	
	if( (result = ivcp_gsensor_wakeup_disable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_wakeup_disable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void get_gsensor_wakeup_status()
{
	mrm_err result;
	
	char status;
	
	if( (result = ivcp_gsensor_get_wakeup_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_get_wakeup_status fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Gsensor Wakeup is %s\n", status ? "Enable" : "Disable" );
}

void gsensor_get_wakeup_threshold()
{
	mrm_err result;
	
	int mg;
		
	if( (result = ivcp_gsensor_get_wakeup_threshold(&mg)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_get_wakeup_threshold fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Current Gsensor Wakeup Threshold is %d mg\n", mg);
}

void gsensor_set_wakeup_threshold()
{
	printf("Threshold (mg) :");	
	int mg;
	scanf("%d", &mg);
	
	WAIT_ENTER();
	
	if( mg <0 || mg>16000 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;

	if( (result = ivcp_gsensor_set_wakeup_threshold(mg)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_gsensor_set_wakeup_threshold fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void gsensor_main()
{
	int op;
	
	if( gsensor_init() < 0 )
		return ;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
			
		switch(op)
		{
			case GSENSOR_ENABLE:
				gsensor_enable();
				break;
			case GSENSOR_DISABLE:
				gsensor_disable();
				break;
			case GET_GSENSOR_STATUS:
				get_gsensor_status();
				break;
			case GSENSOR_READ:
				read_gsensor();
				break;				
			case GET_GSENSOR_RESOLUTION:
				gsensor_get_resolution();
				break;
			case SET_GSENSOR_RESOLUTION:
				gsensor_set_resolution();
				break;
			case GSENSOR_WAKEUP_ENABLE:
				gsensor_wakeup_enable();
				break;
			case GSENSOR_WAKEUP_DISABLE:
				gsensor_wakeup_disable();
				break;
			case GET_GSENSOR_WAKEUP_STATUS:
				get_gsensor_wakeup_status();
				break;
			case GET_GSENSOR_WAKEUP_THRESHOLD:
				gsensor_get_wakeup_threshold();
				break;
			case SET_GSENSOR_WAKEUP_THRESHOLD:
				gsensor_set_wakeup_threshold();
				break;	
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
