#include <stdio.h>
#include <unistd.h>

#include "common.h"
#include "ivcp/library.h"
#include "ivcp/peripheral_control.h"

enum peripheral_function{
	GET_POWER_STATUS = 1,
	SET_POWER_STATUS,
	ENABLE_WWAN_WAKEUP,
	DISABLE_WWAN_WAKEUP,
	GET_WWAN_WAKEUP_STATUS,
	GET_SIM_STATUS,
	SET_SIM_STATUS,
	GET_REAR_VIEW,
	SET_REAR_VIEW,
	ENABLE_AUTO_REAR_VIEW,
	DISABLE_AUTO_REAR_VIEW,
	GET_AUTO_REAR_VIEW_STATUS,
	GET_VOICE_INPUT,
	SET_VOICE_INPUT
};

static const char *power_control_module_name[] = {
	"WWAN",
	"WIFI",
	"GPS"
};

int module_control_supported[IVCP_PERIPHERAL_MIC_SWITCH] = {0};

static void show_welcome_title(void)
{
	printf("IVCP SDK Sample -- library version:%s\n", library_version);
	printf(">> Peripheral Control Menu\n\n");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) Get Module Power Status\n", GET_POWER_STATUS);
	printf("%d) Set Module Power Status\n", SET_POWER_STATUS);
	printf("%d) Enable WWAN Wakeup\n", ENABLE_WWAN_WAKEUP);
	printf("%d) Disable WWAN Wakeup\n", DISABLE_WWAN_WAKEUP);
	printf("%d) Get WWAN Wakueup Status\n", GET_WWAN_WAKEUP_STATUS);
	printf("%d) Get SIM Status\n", GET_SIM_STATUS);
	printf("%d) Set SIM Status\n", SET_SIM_STATUS);
	printf("%d) Get Rearview Status\n", GET_REAR_VIEW);
	printf("%d) Set Rearview Status\n", SET_REAR_VIEW);
	printf("%d) Enable auto Rearview detection\n", ENABLE_AUTO_REAR_VIEW);
	printf("%d) Disable auto Rearview detection\n", DISABLE_AUTO_REAR_VIEW);
	printf("%d) Get auto Rearview detection status\n", GET_AUTO_REAR_VIEW_STATUS);
	printf("%d) Get voice input status\n", GET_VOICE_INPUT);
	printf("%d) Set voice input\n", SET_VOICE_INPUT);

	printf("\nEnter your choice: ");
}

int peripheral_init()
{
	static int init_once = 0;

	if( init_once )
		return 0;

	mrm_err result;

	char supported;

	for(int i = IVCP_PERIPHERAL_WWAN_POWER; i <= IVCP_PERIPHERAL_MIC_SWITCH; i++)
	{
		if( (result = ivcp_peripheral_control_available((ivcp_peripheral_control_type)i, &supported)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_peripheral_control_available fail. error code=0x%08x\n", result);
			return -1;
		}
		module_control_supported[i] = supported ? 1 : 0;
	}

	init_once = 1;

	return 0;
}

void get_power_status()
{
	mrm_err result;

	char status;

	for( int i = IVCP_PERIPHERAL_PID_WWAN; i <= IVCP_PERIPHERAL_PID_GPS; i++)
	{
		if( module_control_supported[(int)i] )
		{
			if( (result = ivcp_peripheral_get_power_status((ivcp_peripheral_power_id)i, &status)) != MRM_ERR_NO_ERROR )
			{
				printf("ivcp_peripheral_get_power_status fail. error code=0x%08x\n", result);
				return;
			}

			printf("%s: %s\n", power_control_module_name[i], status ? "ON" : "OFF" );
		}
		else
		{
			printf("%s: -\n", power_control_module_name[i]);
		}
	}
}

void set_power_status()
{
	mrm_err result;

	printf("Select Module:\n");
	for(int i = IVCP_PERIPHERAL_PID_WWAN; i <= IVCP_PERIPHERAL_PID_GPS; i++)
		printf("%d) %s\n", (int)i, power_control_module_name[i]);
	printf("Enter you choice:");

	int id;
	scanf("%d", &id);

	WAIT_ENTER();

	if( id < IVCP_PERIPHERAL_PID_WWAN || id > IVCP_PERIPHERAL_PID_GPS)
	{
		printf("Error range\n");
		return;
	}

	int s;

	printf("Enable 1 ? Disable 0:");
	scanf("%d", &s);

	WAIT_ENTER();

	if( s < 0 || s >1)
	{
		printf("Error range\n");
		return;
	}

	if( s )
	{
		if( (result = ivcp_peripheral_power_on((ivcp_peripheral_power_id)id)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_peripheral_power_on fail. error code=0x%08x\n", result);
			return;
		}
	}
	else
	{
		if( (result = ivcp_peripheral_power_off((ivcp_peripheral_power_id)id)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_peripheral_power_off fail. error code=0x%08x\n", result);
			return;
		}
	}

	printf("Success\n");
}

void wwan_wakeup_enable()
{
	mrm_err result;

	if( (result = ivcp_peripheral_wwan_wakeup_enable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_wwan_wakeup_enable fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void wwan_wakeup_disable()
{
	mrm_err result;

	if( (result = ivcp_peripheral_wwan_wakeup_disable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_wwan_wakeup_disable fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void get_wwan_wakeup_status()
{
	mrm_err result;

	char status;

	if( (result = ivcp_peripheral_get_wwan_wakeup_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_get_wwan_wakeup_status fail. error code=0x%08x\n", result);
		return;
	}

	printf("The WWAN Wakueup is %s\n", status ? "Enable" : "Disable" );
}

void get_sim()
{
	if( module_control_supported[IVCP_PERIPHERAL_SIM_SWITCH] == 0)
	{
		printf("The platform not supported control this type.\n");
		return;
	}

	mrm_err result;

	ivcp_peripheral_sim_id sid;
	const char *sid_string[] = { "SIM0","SIM1"};

	if( (result = ivcp_peripheral_get_sim(&sid)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_get_sim fail. error code=0x%08x\n", result);
		return;
	}

	printf("The Current SIM is %s\n", sid_string[(int)sid]);
}

void set_sim()
{
	if( module_control_supported[IVCP_PERIPHERAL_SIM_SWITCH] == 0)
	{
		printf("The platform not supported control this type.\n");
		return;
	}

	printf("SIM :\n");
	printf("0) SIM0\n");
	printf("1) SIM1\n");

	int id;
	scanf("%d", &id);

	WAIT_ENTER();

	if( id <0 || id>1 )
	{
		printf("Error range\n");
		return;
	}

	mrm_err result;

	if( (result = ivcp_peripheral_set_sim((ivcp_peripheral_sim_id)id)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_set_sim fail. error code=0x%08x\n", result);
		return;
	}


	printf("Success\n");
}

void get_rear_view()
{
	if( module_control_supported[IVCP_PERIPHERAL_REARVIEW_SWITCH] == 0)
	{
		printf("The platform not supported control this type.\n");
		return;
	}

	mrm_err result;

	ivcp_peripheral_rearview_id rid;
	const char *rid_string[] = { "Main","Ext1"};

	if( (result = ivcp_peripheral_get_rearview(&rid)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_get_rearview fail. error code=0x%08x\n", result);
		return;
	}

	printf("The Current view is %s\n", rid_string[(int)rid]);
}

void set_rear_view()
{
	if( module_control_supported[IVCP_PERIPHERAL_REARVIEW_SWITCH] == 0)
	{
		printf("The platform not supported control this type.\n");
		return;
	}

	printf("View ID :\n");
	printf("0) Main\n");
	printf("1) Externel1\n");
	printf("2) Externel1 (5 Second auto return main view)\n");

	int id;
	scanf("%d", &id);

	WAIT_ENTER();

	if( id <0 || id>2 )
	{
		printf("Error range\n");
		return;
	}

	mrm_err result;

	bool auto_return_main_view = (id == 2 ? true : false);

	if( auto_return_main_view)
		id = 1;

	if( (result = ivcp_peripheral_set_rearview((ivcp_peripheral_rearview_id)id)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_set_rearview fail. error code=0x%08x\n", result);
		return;
	}

	if(auto_return_main_view)
	{
		int countdown_sec = 5;
		while(countdown_sec--)
		{
			printf("%d sec return main..\n", countdown_sec+1);
			usleep(1000000);
		}
		ivcp_peripheral_set_rearview(IVCP_PERIPHERAL_RID_MAIN);
	}

	printf("Success\n");
}

void auto_rearview_enable()
{
	if( module_control_supported[IVCP_PERIPHERAL_REARVIEW_SWITCH] == 0)
	{
		printf("The platform not supported control this type.\n");
		return;
	}

	mrm_err result;

	if( (result = ivcp_peripheral_auto_rearview_enable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_auto_rearview_enable fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void auto_rearview_disable()
{
	if( module_control_supported[IVCP_PERIPHERAL_REARVIEW_SWITCH] == 0)
	{
		printf("The platform not supported control this type.\n");
		return;
	}

	mrm_err result;

	if( (result = ivcp_peripheral_auto_rearview_disable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_auto_rearview_disable fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void get_auto_rearview_status()
{
	if( module_control_supported[IVCP_PERIPHERAL_REARVIEW_SWITCH] == 0)
	{
		printf("The platform not supported control this type.\n");
		return;
	}

	mrm_err result;

	char status;

	if( (result = ivcp_peripheral_get_auto_rearview_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_get_auto_rearview_status fail. error code=0x%08x\n", result);
		return;
	}

	printf("The Auto Rearview Detection is %s\n", status ? "Enable" : "Disable" );
}

void get_voice_input()
{
	if( module_control_supported[IVCP_PERIPHERAL_MIC_SWITCH] == 0)
	{
		printf("The platform not supported control this type.\n");
		return;
	}

	mrm_err result;

	ivcp_peripheral_voice_input type;
	const char *type_string[] = { "Front Panel MIC","MIC-IN", "Bluetooth"};

	if( (result = ivcp_peripheral_get_voice_input(&type)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_get_voice_input fail. error code=0x%08x\n", result);
		return;
	}

	printf("The Current Voice Input is %s\n", type_string[(int)type]);
}

void set_voice_input()
{
	if( module_control_supported[IVCP_PERIPHERAL_MIC_SWITCH] == 0)
	{
		printf("The platform not supported control this type.\n");
		return;
	}

	printf("Voice Input :\n");
	printf("0) Front Panel MIC\n");
	printf("1) MIC-IN\n");
	printf("2) Bluetooth\n");

	int id;
	scanf("%d", &id);

	WAIT_ENTER();

	if( id <0 || id>2 )
	{
		printf("Error range\n");
		return;
	}

	mrm_err result;

	if( (result = ivcp_peripheral_set_voice_input((ivcp_peripheral_voice_input)id)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_peripheral_set_voice_input fail. error code=0x%08x\n", result);
		return;
	}


	printf("Success\n");
}

void peripheral_main()
{
	int op;

	if( peripheral_init() < 0 )
		return ;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;

		switch(op)
		{
			case GET_POWER_STATUS:
				get_power_status();
				break;
			case SET_POWER_STATUS:
				set_power_status();
				break;
			case ENABLE_WWAN_WAKEUP:
				wwan_wakeup_enable();
				break;
			case DISABLE_WWAN_WAKEUP:
				wwan_wakeup_disable();
				break;
			case GET_WWAN_WAKEUP_STATUS:
				get_wwan_wakeup_status();
				break;
			case GET_SIM_STATUS:
				get_sim();
				break;
			case SET_SIM_STATUS:
				set_sim();
				break;
			case GET_REAR_VIEW:
				get_rear_view();
				break;
			case SET_REAR_VIEW:
				set_rear_view();
				break;
			case ENABLE_AUTO_REAR_VIEW:
				auto_rearview_enable();
				break;
			case DISABLE_AUTO_REAR_VIEW:
				auto_rearview_disable();
				break;
			case GET_AUTO_REAR_VIEW_STATUS:
				get_auto_rearview_status();
				break;
			case GET_VOICE_INPUT:
				get_voice_input();
				break;
			case SET_VOICE_INPUT:
				set_voice_input();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
