#include <stdio.h>

#include "common.h"
#include "ivcp/library.h"
#include "ivcp/power_management.h"

enum page_function{
	GET_IGNITION_STATUS = 1,
	IGNITION_WAKEUP_ENABLE,
	IGNITION_WAKEUP_DISABLE,
	GET_IGNITION_WAKEUP_STATUS,
	GET_VOLTAGE,
	GET_KEEP_ALIVEMODE,
	SET_KEEP_ALIVEMODE,
	GET_AT_MODE,
	SET_AT_MODE,
	GET_EVENT_DELAY,
	SET_EVENT_DELAY,
	GET_LAST_WAKEUP_SOURCE,
	GET_LOW_POWER_PROTECTION_STATUS,
	SET_LOW_POWER_PROTECTION,
	GET_LOW_POWER_PROTECTION_THRESHOLD,
	SET_LOW_POWER_PROTECTION_THRESHOLD,
	RESET_LOW_POWER_PROTECTION_THRESHOLD,
	GET_POWER_MODE,
	SET_POWER_MODE,
	GET_SHUTDOWN_MASK,
	SET_SHUTDOWN_MASK,
	GET_FORCE_SHUTDOWN,
	FORCE_SHUTDOWN_ENABLE,
	FORCE_SHUTDOWN_DISABLE,
	GET_FORCE_SHUTDOWN_DELAY,
	SET_FORCE_SHUTDOWN_DELAY
};

static const char *wakeup_source_string[] = {
	"Power Button",
	"Ignition",
	"WWAN",
	"Gsensor",
	"DI1",
	"DI2",
	"Alarm",
	"Hotkey",
	"DI3",
	"DI4",
	"Keep Alive Mode",
	"AT Mode",
	"Reset"
};
	
static void show_welcome_title(void)
{
	printf("IVCP SDK Sample -- library version:%s\n", library_version);
	printf(">> Power Management Menu\n\n");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) Get Ignition Status\n", GET_IGNITION_STATUS);
	printf("%d) Enable Ignition Wakeup\t", IGNITION_WAKEUP_ENABLE);
	printf("%d) Disable Ignition Wakeup\n", IGNITION_WAKEUP_DISABLE);
	printf("%d) Get Ignition Wakeup Status\n", GET_IGNITION_WAKEUP_STATUS);
	printf("%d) Get Current Voltage\n", GET_VOLTAGE);
	printf("%d) Get Keep-Alive Mode Status\t", GET_KEEP_ALIVEMODE);
	printf("%d) Set Keep-Alive Mode\n",SET_KEEP_ALIVEMODE);
	printf("%d) Get AT Mode Status\t", GET_AT_MODE);
	printf("%d) Set AT Mode Status\n", SET_AT_MODE);
	printf("%d) Get Event Delay Time\t", GET_EVENT_DELAY);
	printf("%d) Set Event Delay Time\n", SET_EVENT_DELAY);
	printf("%d) Get Last Wakeup Source\n",GET_LAST_WAKEUP_SOURCE);
	printf("%d) Get Low Power Protection Status\n",GET_LOW_POWER_PROTECTION_STATUS);
	printf("%d) Set Low Power Protection\n",SET_LOW_POWER_PROTECTION);
	printf("%d) Get Low Power Protection Threshold\t",GET_LOW_POWER_PROTECTION_THRESHOLD);
	printf("%d) Set Low Power Protection Threshold\n",SET_LOW_POWER_PROTECTION_THRESHOLD);
	printf("%d) Reset Low Power Protection Threshold\n",RESET_LOW_POWER_PROTECTION_THRESHOLD);	
	printf("%d) Get Power Mode\t", GET_POWER_MODE);
	printf("%d) Set Power Mode(12V,24V,48V)\n", SET_POWER_MODE);
	printf("%d) Get Shutdown Mask\t", GET_SHUTDOWN_MASK);
	printf("%d) Set Shutdown Mask\n", SET_SHUTDOWN_MASK);
	printf("%d) Get Force Shutdown Status\t", GET_FORCE_SHUTDOWN);
	printf("%d) Enable Force Shutdonw\t", FORCE_SHUTDOWN_ENABLE);
	printf("%d) Disable Force Shutdonw\n", FORCE_SHUTDOWN_DISABLE);
	printf("%d) Get Force Shutdonw Delay\t", GET_FORCE_SHUTDOWN_DELAY);
	printf("%d) Set Force Shutdonw Delay\n", SET_FORCE_SHUTDOWN_DELAY);
		
	printf("\nEnter your choice: ");
}

void get_ignition_status()
{
	mrm_err result;
	
	char status;
	
	if( (result = ivcp_pm_get_ignition_status( &status )) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_ignition_status. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Ignition is %s\n", status ? "ON" : "OFF");
}

void ignition_wakeup_enable()
{
	mrm_err result;
	
	if( (result = ivcp_pm_ignition_wakeup_enable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_ignition_wakeup_enable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void ignition_wakeup_disable()
{
	mrm_err result;
	
	if( (result = ivcp_pm_ignition_wakeup_disable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_ignition_wakeup_disable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void get_ignition_wakeup_status()
{
	mrm_err result;
	
	char status;
	
	if( (result = ivcp_pm_get_ignition_wakeup_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_ignition_wakeup_status fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Ignition Wakeup is %s\n", status ? "Enable" : "Disable" );
}

void get_voltage()
{
	mrm_err result;
	
	float vol;
	
	if( (result = ivcp_pm_get_voltage( &vol )) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_voltage. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Current voltage is %.2f volt\n", vol);
}

void get_keep_alive_mode()
{
	mrm_err result;

	char mode;
	
	if( (result = ivcp_pm_get_alive_mode(&mode)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_alive_mode fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Keep Alive Mode is %s\n", mode ? "Enable" : "Disable" );
}

void set_keep_alive_mode()
{
	printf("Mode:\n");
	printf("0) Disable\n");
	printf("1) Enable\n");

	int m;
	scanf("%d", &m);
	
	WAIT_ENTER();
	
	if( m <0 || m>1 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;

	if( (result = ivcp_pm_set_alive_mode(m)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_set_alive_mode fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void get_at_mode()
{
	mrm_err result;

	char mode;
	
	if( (result = ivcp_pm_get_at_mode(&mode)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_at_mode fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("AT Mode is %s\n", mode ? "Enable" : "Disable" );
}

void set_at_mode()
{
	printf("Mode:\n");
	printf("0) Disable\n");
	printf("1) Enable\n");

	int m;
	scanf("%d", &m);
	
	WAIT_ENTER();
	
	if( m <0 || m>1 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;

	if( (result = ivcp_pm_set_at_mode(m)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_set_at_mode fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void get_event_delay()
{
	printf("Event:\n");
	printf("0) Low Voltage\n");
	printf("1) Low Voltage Hard\n");
	printf("2) Ignition ON\n");
	printf("3) Ignotion OFF to Power OFF\n");
	printf("4) Ignotion OFF Hard\n");
	printf("5) Post Boot Power Check\n");
	printf("6) Ignotion OFF to Suspend\n");
	
	int e;
	scanf("%d", &e);
	
	WAIT_ENTER();
	
	if( e <0 || e>6 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;
	
	unsigned int second;
	
	if( (result = ivcp_pm_get_event_delay((ivcp_pm_event)e, &second)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_event_delay fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Delay Time is %d second", second);
}

void set_event_delay()
{
	printf("Event:\n");
	printf("0) Low Voltage\n");
	printf("1) Low Voltage Hard\n");
	printf("2) Ignition ON\n");
	printf("3) Ignotion OFF to Power OFF\n");
	printf("4) Ignotion OFF Hard\n");
	printf("5) Post Boot Power Check\n");
	printf("6) Ignotion OFF to Suspend\n");
	
	int e;
	scanf("%d", &e);
	
	WAIT_ENTER();
	
	if( e <0 || e>6 )
	{
		printf("Error range\n");
		return;
	}
	
	unsigned int s;
	scanf("%u", &s);
	
	WAIT_ENTER();
	
	if( e > 1 )
	{
		if( s <0 || s>18000)
		{
			printf("Error range\n");
			return;
		}
	}
	else
	{
		if( s <0 || s>3600)
		{
			printf("Error range\n");
			return;
		}
	}
	
	mrm_err result;
	
	if( (result = ivcp_pm_set_event_delay((ivcp_pm_event)e, s)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_set_event_delay fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void get_last_wakeup_source()
{
	mrm_err result;
	
	ivcp_pm_wakeup_type source;
	
	if( (result = ivcp_pm_get_last_wakeup_source(&source)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_last_wakeup_source fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Last Wakeup Source from %s\n", wakeup_source_string[(int)source]);
}

void get_lvp_status()
{
	printf("Type :\n");
	printf("0) Pre-boot\n");
	printf("1) Post\n");
	
	int t;
	scanf("%d", &t);
	
	WAIT_ENTER();
	
	if( t <0 || t>1 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;
	char status;

	if( t )
	{
		if( (result = ivcp_pm_lvp_postboot_get_status(&status)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_pm_lvp_postboot_get_status fail. error code=0x%08x\n", result);
			return;
		}
	}
	else
	{
		if( (result = ivcp_pm_lvp_preboot_get_status(&status)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_pm_lvp_preboot_get_status fail. error code=0x%08x\n", result);
			return;
		}
	}

	printf("The %s Low Power Protection is %s\n", t ? "Post-boot" : "Pre-boot",  status ? "Enable" : "Disable");
}

void set_lvp_status()
{
	printf("Type :\n");
	printf("0) Pre-boot\n");
	printf("1) Post\n");
	
	int t;
	scanf("%d", &t);
	
	WAIT_ENTER();
	
	if( t <0 || t>1 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;
	int s;
	printf("0) Disable\n");
	printf("1) Enable\n");
	scanf("%d", &s);
	
	WAIT_ENTER();
	
	if( s <0 || s>1 )
	{
		printf("Error range\n");
		return;
	}

	if( t )
	{
		if( s )
		{
			if( (result = ivcp_pm_lvp_postboot_enable()) != MRM_ERR_NO_ERROR )
			{
				printf("ivcp_pm_lvp_postboot_enable fail. error code=0x%08x\n", result);
				return;
			}
		}
		else
		{
			if( (result = ivcp_pm_lvp_postboot_disable()) != MRM_ERR_NO_ERROR )
			{
				printf("ivcp_pm_lvp_postboot_disable fail. error code=0x%08x\n", result);
				return;
			}		
		}
	}
	else
	{
		if( s )
		{
			if( (result = ivcp_pm_lvp_preboot_enable()) != MRM_ERR_NO_ERROR )
			{
				printf("ivcp_pm_lvp_preboot_enable fail. error code=0x%08x\n", result);
				return;
			}
		}
		else
		{
			if( (result = ivcp_pm_lvp_preboot_disable()) != MRM_ERR_NO_ERROR )
			{
				printf("ivcp_pm_lvp_preboot_disable fail. error code=0x%08x\n", result);
				return;
			}		
		}
	}

	printf("Success\n");
}

void get_lvp_threshold()
{
	printf("Type :\n");
	printf("0) Pre-boot\n");
	printf("1) Post\n");
	
	int t;
	scanf("%d", &t);
	
	WAIT_ENTER();
	
	if( t <0 || t>1 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;
	float vol;
	
	if( t )
	{
		if( (result = ivcp_pm_get_lvp_postboot_threshold(&vol)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_pm_get_lvp_postboot_threshold fail. error code=0x%08x\n", result);
			return;
		}
	}
	else
	{
		if( (result = ivcp_pm_get_lvp_preboot_threshold(&vol)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_pm_get_lvp_preboot_threshold fail. error code=0x%08x\n", result);
			return;
		}
	}

	printf("The %s Low Power Protection is %.2f volt\n", t ? "Post-boot" : "Pre-boot", vol);
}

static float voltage_range_min = 0, voltage_range_max = 0, voltage_range_default =0;

int get_lvp_range()
{
	static int init_once = 0;
	
	if( init_once )
		return 0;
	
	mrm_err result;
		
	if( (result = ivcp_pm_get_lvp_range(&voltage_range_min, &voltage_range_max, &voltage_range_default)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_lvp_range fail. error code=0x%08x\n", result);
		return -1;
	}
	
	init_once = 1;
	
	return 0;
}

void set_lvp_threshold()
{
	if(get_lvp_range())
		return;
	
	printf("Type :\n");
	printf("0) Pre-boot\n");
	printf("1) Post\n");
	
	int t;
	scanf("%d", &t);
	
	WAIT_ENTER();
	
	if( t <0 || t>1 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;
	float v;
	printf("Threshold (min:%.2f max:%.2f default:%.2f):",voltage_range_min, voltage_range_max, voltage_range_default);
	scanf("%f", &v);
	
	WAIT_ENTER();
	
	if( v < voltage_range_min || v>voltage_range_max )
	{
		printf("Error range\n");
		return;
	}

	if( t )
	{
		if( (result = ivcp_pm_set_lvp_postboot_threshold(v)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_pm_set_lvp_postboot_threshold fail. error code=0x%08x\n", result);
			return;
		}
	}
	else
	{
		if( (result = ivcp_pm_set_lvp_preboot_threshold(v)) != MRM_ERR_NO_ERROR )
		{
			printf("ivcp_pm_set_lvp_preboot_threshold fail. error code=0x%08x\n", result);
			return;
		}
	}

	printf("Success\n");
}

void reset_lvp_threshold()
{	
	mrm_err result;

	if( (result = ivcp_pm_reset_lvp_threshold()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_reset_lvp_threshold fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void get_power_mode()
{
	mrm_err result;

	char mode;
	
	if( (result = ivcp_pm_get_power_mode(&mode)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_power_mode fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Power Mode is %s\n", mode == IVCP_POWER_MODE_48V ? "48V" : (mode == IVCP_POWER_MODE_24V ? "24V" : "12V") );
}

void set_power_mode()
{
	printf("Mode:\n");
	printf("1) 48V\n");
	printf("2) 24V\n");
	printf("3) 12V\n");

	int m;
	scanf("%d", &m);
	
	WAIT_ENTER();
	
	if( m <1 || m>3 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;

	if( (result = ivcp_pm_set_power_mode(m)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_set_power_mode fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void get_shutdown_mask()
{
	mrm_err result;
	
	char status;
		
	printf("Select Mask:\n");
	printf("0) Power Button\n");
	printf("1) Ignition\n");

	int m;
	scanf("%d", &m);
	
	WAIT_ENTER();
	
	if( m <0 || m>1 )
	{
		printf("Error range\n");
		return;
	}
		
	if( (result = ivcp_pm_get_shutdown_mask((ivcp_pm_shutdown_mask)m, &status)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_shutdown_mask fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Shutdown mask status is %s\n", status == 0 ? "Disable" : "Enable");
}

void set_shutdown_mask()
{
	printf("Select Mask:\n");
	printf("0) Power Button\n");
	printf("1) Ignition\n");

	int m;
	scanf("%d", &m);
	
	WAIT_ENTER();
	
	if( m <0 || m>1 )
	{
		printf("Error range\n");
		return;
	}
	
	printf("Mode:\n");
	printf("0) Disable\n");
	printf("1) Enable\n");

	int s;
	scanf("%d", &s);
	
	WAIT_ENTER();
	
	if( s <0 || s>1 )
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;

	if( (result = ivcp_pm_set_shutdown_mask((ivcp_pm_shutdown_mask)m, (char)s)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_set_shutdown_mask fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void force_shutdown_enable()
{
	mrm_err result;
	
	if( (result = ivcp_pm_force_shutdown_enable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_force_shutdown_enable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void force_shutdown_disable()
{
	mrm_err result;
	
	if( (result = ivcp_pm_force_shutdown_disable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_force_shutdown_disable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void get_force_shutdown_status()
{
	mrm_err result;
	
	char status;
	
	if( (result = ivcp_pm_get_force_shutdown_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_force_shutdown_status fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Force shutdown is %s\n", status ? "Enable" : "Disable" );
}

void get_force_shutdown_delay()
{	
	mrm_err result;
	
	unsigned int second;
	
	if( (result = ivcp_pm_get_force_shutdown_delay(&second)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_get_force_shutdown_delay fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The ForceShutdown Delay Time is %d second", second);
}

void set_force_shutdown_delay()
{
	unsigned int s;
	scanf("%u", &s);
	
	WAIT_ENTER();
	
	if( s <0 || s>3600)
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;
		
	if( (result = ivcp_pm_set_force_shutdown_delay(s)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_pm_set_force_shutdown_delay fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void power_management_main()
{
	int op;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
			
		switch(op)
		{
			case GET_IGNITION_STATUS:
				get_ignition_status();
				break;
			case IGNITION_WAKEUP_ENABLE:
				ignition_wakeup_enable();
				break;
			case IGNITION_WAKEUP_DISABLE:
				ignition_wakeup_disable();
				break;
			case GET_IGNITION_WAKEUP_STATUS:
				get_ignition_wakeup_status();
				break;
			case GET_VOLTAGE:
				get_voltage();
				break;
			case GET_KEEP_ALIVEMODE:
				 get_keep_alive_mode();
				break;
			case SET_KEEP_ALIVEMODE:
				set_keep_alive_mode();
				break;
			case GET_AT_MODE:
				get_at_mode();
				break;
			case SET_AT_MODE:
				set_at_mode();
				break;
			case GET_EVENT_DELAY:
				get_event_delay();
				break;
			case SET_EVENT_DELAY:
				set_event_delay();
				break;
			case GET_LAST_WAKEUP_SOURCE:
				get_last_wakeup_source();
				break;
			case GET_LOW_POWER_PROTECTION_STATUS:
				get_lvp_status();
				break;
			case SET_LOW_POWER_PROTECTION:
				set_lvp_status();
				break;
			case GET_LOW_POWER_PROTECTION_THRESHOLD:
				get_lvp_threshold();
				break;
			case SET_LOW_POWER_PROTECTION_THRESHOLD:
				set_lvp_threshold();
				break;
			case RESET_LOW_POWER_PROTECTION_THRESHOLD:
				reset_lvp_threshold();
				break;
			case GET_POWER_MODE:
				get_power_mode();
				break;
			case SET_POWER_MODE:
				set_power_mode();
				break;
			case GET_SHUTDOWN_MASK:
				get_shutdown_mask();
				break;
			case SET_SHUTDOWN_MASK:
				set_shutdown_mask();
				break;
			case GET_FORCE_SHUTDOWN:
				get_force_shutdown_status();
				break;
			case FORCE_SHUTDOWN_ENABLE:
				force_shutdown_enable();
				break;
			case FORCE_SHUTDOWN_DISABLE:
				force_shutdown_disable();
				break;
			case GET_FORCE_SHUTDOWN_DELAY:
				get_force_shutdown_delay();
				break;
			case SET_FORCE_SHUTDOWN_DELAY:
				set_force_shutdown_delay();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
