#include <stdio.h>

#include "common.h"
#include "ivcp/library.h"
#include "ivcp/psensor.h"

enum page_function{
	PSENSOR_ENABLE = 1,
	PSENSOR_DISABLE,
	GET_PSENSOR_STATUS,
	PSENSOR_READ
};

static void show_welcome_title(void)
{
	printf("IVCP SDK Sample -- library version:%s\n", library_version);
	printf(">> Psensor Control Menu\n\n");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) Enable Psensor\n", PSENSOR_ENABLE);
	printf("%d) Disable Psensor\n", PSENSOR_DISABLE);
	printf("%d) Get Psensor Status\n", GET_PSENSOR_STATUS);
	printf("%d) Read Pressure\n", PSENSOR_READ);
		
	printf("\nEnter your choice: ");
}

int psensor_init()
{
	static int init_once = 0;
	
	if( init_once )
		return 0;
	
	mrm_err result;
	
	char supported;
	
	if( (result = ivcp_psensor_available(&supported)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_psensor_available fail. error code=0x%08x\n", result);
		return -1;
	}
	
	if( !supported )
	{
		printf("Psensor not supported\n");
		WAIT_ENTER();
	}
	
	init_once = 1;
	
	return 0;
}

void psensor_enable()
{
	mrm_err result;
	
	if( (result = ivcp_psensor_enable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_psensor_enable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void psensor_disable()
{
	mrm_err result;
	
	if( (result = ivcp_psensor_disable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_psensor_disable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void get_psensor_status()
{
	mrm_err result;
	
	char status;
	
	if( (result = ivcp_psensor_get_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_psensor_get_status fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The Psensor is %s\n", status ? "Enable" : "Disable" );
}


void read_psensor()
{
	mrm_err result;
	
	int mbar;

	if( (result = ivcp_psensor_get_pressure( &mbar )) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_psensor_get_pressure fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The pressure is %d mBar\n", mbar);
}

void psensor_main()
{
	int op;
	
	if( psensor_init() < 0 )
		return ;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
			
		switch(op)
		{
			case PSENSOR_ENABLE:
				psensor_enable();
				break;
			case PSENSOR_DISABLE:
				psensor_disable();
				break;
			case GET_PSENSOR_STATUS:
				get_psensor_status();
				break;
			case PSENSOR_READ:
				read_psensor();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
