#include <stdio.h>

#include "common.h"
#include "ivcp/library.h"
#include "ivcp/watchdog.h"

enum page_function{
	WATCHDOG_ENABLE = 1,
	WATCHDOG_DISABLE,
	WATCHDOG_GET_TIME,
	WATCHDOG_SET_TIME,
	WATCHDOG_GET_CURRENT_TIME,
	WATCHDOG_TRIGGER
};

static void show_welcome_title(void)
{
	printf("IVCP SDK Sample -- library version:%s\n", library_version);
	printf(">> Watchdog Control Menu\n");
	printf(" (CAUTION if watchdog enable out of time to trigger, System will reboot)\n\n");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) Watchdog Enable\n", WATCHDOG_ENABLE);
	printf("%d) Watchdog Disable\n", WATCHDOG_DISABLE);
	printf("%d) Watchdog Get time\n", WATCHDOG_GET_TIME);
	printf("%d) Watchdog Set time\n", WATCHDOG_SET_TIME);
	printf("%d) Watchdog Get Current time\n", WATCHDOG_GET_CURRENT_TIME);
	printf("%d) Watchdog Trigger\n", WATCHDOG_TRIGGER);
		
	printf("\nEnter your choice: ");
}

void watchdog_enable()
{
	mrm_err result;
	
	if( (result = ivcp_watchdog_enable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_watchdog_enable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void watchdog_disable()
{
	mrm_err result;
	
	if( (result = ivcp_watchdog_disable()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_watchdog_disable fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void watchdog_trigger()
{
	mrm_err result;
	
	if( (result = ivcp_watchdog_trigger()) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_watchdog_trigger fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("Success\n");
}

void watchdog_get_time()
{
	mrm_err result;
	
	unsigned short countdown_time;
	
	if( (result = ivcp_watchdog_get_time(&countdown_time)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_watchdog_trigger fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The watchdog time is %hu second.\n", countdown_time);
}

void watchdog_set_time()
{
	printf("Watchdog time (0-65535):");
	unsigned short s;
	scanf("%hu", &s);
	
	WAIT_ENTER();
	
	if( s > 65535)
	{
		printf("Error range\n");
		return;
	}
	
	mrm_err result;

	if( (result = ivcp_watchdog_set_time(s)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_watchdog_set_time fail. error code=0x%08x\n", result);
		return;
	}


	printf("Success\n");
}

void watchdog_get_current_time()
{
	mrm_err result;
	
	unsigned short countdown_time;
	
	if( (result = ivcp_watchdog_get_current_time(&countdown_time)) != MRM_ERR_NO_ERROR )
	{
		printf("ivcp_watchdog_trigger fail. error code=0x%08x\n", result);
		return;
	}
	
	printf("The current watchdog time is %hu second.\n", countdown_time);
}

void watchdog_main()
{
	int op;
	
	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;

		switch(op)
		{
			case WATCHDOG_ENABLE:
				watchdog_enable();
				break;
			case WATCHDOG_DISABLE:
				watchdog_disable();
				break;
			case WATCHDOG_GET_TIME:
				watchdog_get_time();
				break;
			case WATCHDOG_SET_TIME:
				watchdog_set_time();
				break;
			case WATCHDOG_GET_CURRENT_TIME:
				watchdog_get_current_time();
				break;
			case WATCHDOG_TRIGGER:
				watchdog_trigger();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
