#include <stdio.h>

#include "common.h"
#include "function.h"
#include "sdp/sdp.h"

char library_version[SDP_MAXIMUM_LIBRARY_STRING_LENGTH]={0};
char platform_name[SDP_MAXIMUM_PLATFORM_STRING_LENGTH]={0};

static const char *page_name[] = {
	"Firmware management",
	"Backlight",
	"Hotkey",
	"Peripheral control",
	"Sensor"
};

static const int page_function_supported[] = {
	SDP_SAMPLE_CODE_FUNCTION_FIRMWARE,
	SDP_SAMPLE_CODE_FUNCTION_BACKLIGHT,
	SDP_SAMPLE_CODE_FUNCTION_HOTKEY,
	SDP_SAMPLE_CODE_FUNCTION_PERIPHERAL_CONTROL,
	SDP_SAMPLE_CODE_FUNCTION_SENSOR,
	SDP_SAMPLE_CODE_MAX_FUNCTION_PAGE
};

static void show_welcome_title(void)
{
	printf("Smart Display SDK Sample -- library version:%s\n", library_version);
	printf("                         -- platform:%s\n", platform_name);
	printf(">> Main Menu\n\n");
}

static void show_menu(void)
{
	printf("0) exit\n");
	
	for (int i = 0; i < SDP_SAMPLE_CODE_MAX_FUNCTION_PAGE - 1; i++)
	{
		if (page_function_supported[i] >= 0)
			printf("%d) %s\n", i + 1, page_name[i]);
		else
			break;
	}
	
	printf("\nEnter your choice: ");
}

int main(int argc, char *argv[])
{
	mrm_err result;
	int op;

	if( (result = sdp_init()) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_init fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}
	
	if( (result = sdp_get_version(library_version)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_get_version fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}
	
	if( (result = sdp_get_platform_name(platform_name)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_get_platform_name fail. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	} 

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		show_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;

		switch(op)
		{
			case SDP_SAMPLE_CODE_FUNCTION_FIRMWARE:
				firmware_main();
				break;
			case SDP_SAMPLE_CODE_FUNCTION_BACKLIGHT:
				backlight_main();
				break;
			case SDP_SAMPLE_CODE_FUNCTION_HOTKEY:
				hotkey_main();
				break;
			case SDP_SAMPLE_CODE_FUNCTION_PERIPHERAL_CONTROL:
				peripheral_main();
				break;
			case SDP_SAMPLE_CODE_FUNCTION_SENSOR:
				sensor_main();
				break;
			default:
				printf("Unknown choice!\n");
				printf("\nPress ENTER to continue. ");
				WAIT_ENTER();
				break;
		}
	}

	if( (result = sdp_deinit()) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_deinit. error code=0x%08x\n", result);
		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();
		return -1;
	}

	return 0;
}


