#include <stdio.h>

#include "common.h"
#include "sdp/library.h"
#include "sdp/firmware.h"

enum firmware_function{
	FIRMWARE_GET_VERSION = 1,
	FIRMWARE_LOAD_DEFAULT,
	FIRMWARE_SAVE_DEFAULT,
	FIRMWARE_RESET
};

static void show_welcome_title(void)
{
	printf("Smart Display SDK Sample -- library version:%s\n", library_version);
	printf(">> Firmware Menu\n\n");
}

void firmware_menu()
{
	printf("0) back\n");

	printf("%d) Get firmware version\n", FIRMWARE_GET_VERSION);
	printf("%d) Load configure to default\n", FIRMWARE_LOAD_DEFAULT);
	printf("%d) Save configure to default\n", FIRMWARE_SAVE_DEFAULT);
	printf("%d) Reset configure to default\n", FIRMWARE_RESET);
	
	printf("\nEnter your choice: ");
}

void get_firmware_version()
{
	mrm_err result;
	char fw_version[SDP_MAXIMUM_FIRMWARE_VERSION_LENGTH];

	if( (result = sdp_firmware_get_version(fw_version)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_firmware_get_version fail. error code=0x%08x\n", result);
		return;
	}

	printf("Firmware version is: %s\n", fw_version);
}

void load_default_config()
{
	mrm_err result;

	if( (result = sdp_firmware_load_default()) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_firmware_load_default fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void save_default_config()
{
	mrm_err result;

	if( (result = sdp_firmware_save_default()) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_firmware_save_default fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void reset_firmware()
{
	mrm_err result;

	if( (result = sdp_firmware_reset()) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_firmware_reset fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}


void firmware_main()
{
	int op;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		firmware_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;

		switch(op)
		{
			case FIRMWARE_GET_VERSION:
				get_firmware_version();
				break;
			case FIRMWARE_LOAD_DEFAULT:
				load_default_config();
				break;
			case FIRMWARE_SAVE_DEFAULT:
				save_default_config();
				break;
			case FIRMWARE_RESET:
				reset_firmware();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
