#include <stdio.h>

#include "common.h"
#include "sdp/library.h"
#include "sdp/hotkey.h"

enum hotkey_function{
	HOTKEY_READ = 1,
	HOTKEY_GET_BRIGHTNESS,
	HOTKEY_SET_BRIGHTNESS
};

static int key_number = 0;
void hotkey_notify_callback( sdp_keys_value_t *keys );

static void show_welcome_title(void)
{
	printf("Smart Display SDK Sample -- library version:%s\n", library_version);
	printf(">> Hotkey Menu\n\n");
}

void hotkey_menu()
{
	printf("0) back\n");

	printf("%d) Read current hotkey status\n", HOTKEY_READ);
	printf("%d) Get hotkey brightness\n", HOTKEY_GET_BRIGHTNESS);
	printf("%d) Set hotkey brightness\n", HOTKEY_SET_BRIGHTNESS);
	
	printf("\nEnter your choice: ");
}

int hotkey_init()
{
	mrm_err result;
	
	if( (result = sdp_hotkey_get_number(&key_number)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_hotkey_set_event_callback fail. error code=0x%08x\n", result);
		return -1;
	}
	
	if( (result = sdp_hotkey_set_event_callback(hotkey_notify_callback)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_hotkey_set_event_callback fail. error code=0x%08x\n", result);
		return -1;
	}
	
	if( (result = sdp_hotkey_enable_callback()) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_hotkey_enable_callback fail. error code=0x%08x\n", result);
		return -1;
	}

	return 0;
}

int hotkey_deinit()
{
	mrm_err result;
		
	if( (result = sdp_hotkey_disable_callback()) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_hotkey_disable_callback fail. error code=0x%08x\n", result);
		return -1;
	}
	
	if( (result = sdp_hotkey_clear_event_callback()) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_hotkey_clear_event_callback fail. error code=0x%08x\n", result);
		return -1;
	}

	return 0;
}

void hotkey_notify_callback( sdp_keys_value_t *keys )
{
	printf("Hotkey status) 1 2 3 4 5 6 7 8\n");
	printf("               ");
	for(int i=0;i<key_number;i++)
		printf("%d ", keys->key[i] );
	for(int i=key_number;i<8;i++)
		printf("- ");
	printf("\n");
}

void read_hotkey()
{
	mrm_err result;

	sdp_keys_value_t hotkeys;

	if( (result = sdp_hotkey_read(&hotkeys)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_hotkey_read fail. error code=0x%08x\n", result);
		return;
	}

	hotkey_notify_callback(&hotkeys);
}

void get_hotkey_brightness()
{
	mrm_err result;
	unsigned char brightness;

	if( (result = sdp_hotkey_get_brightness(&brightness)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_hotkey_get_brightness fail. error code=0x%08x\n", result);
		return;
	}

	int b = brightness;
	printf("Brightness is %d %%\n", b);
}

void set_hotkey_brightness()
{
	mrm_err result;
	int b;
		
	printf("Hotkey Brightness (0-100):");
	scanf("%d", &b);
	
	WAIT_ENTER();
	
	if( b < 0 || b >100)
	{
		printf("Error range\n");
		return;
	}

	unsigned char brightness = b;
	if( (result = sdp_hotkey_set_brightness(brightness)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_hotkey_set_brightness fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void hotkey_main()
{
	int op;
	
	if(hotkey_init()<0)
		return;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		hotkey_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
			
		switch(op)
		{
			case HOTKEY_READ:
				read_hotkey();
				break;
			case HOTKEY_GET_BRIGHTNESS:
				get_hotkey_brightness();
				break;
			case HOTKEY_SET_BRIGHTNESS:
				set_hotkey_brightness();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
	
	hotkey_deinit();
}
