#include <stdio.h>

#include "common.h"
#include "sdp/library.h"
#include "sdp/speaker.h"
#include "sdp/usb.h"
#include "sdp/reversegear.h"

enum peripheral_function{
	GET_FRONT_END_USB_POWER_STATUS = 1,
	SET_FRONT_END_USB_POWER,
	GET_SPEAKER_MUTE_STATUS,
	SET_SPEAKER_MUTE
};

static void show_welcome_title(void)
{
	printf("Smart Display SDK Sample -- library version:%s\n", library_version);
	printf(">> Peripheral Control Menu\n\n");
}

void peripheral_menu()
{
	printf("0) back\n");

	printf("%d) Get front-end USB power status\n", GET_FRONT_END_USB_POWER_STATUS);
	printf("%d) Set front-end USB power status\n", SET_FRONT_END_USB_POWER);
	printf("%d) Get speaker mute status\n", GET_SPEAKER_MUTE_STATUS);
	printf("%d) Set speaker mute status\n", SET_SPEAKER_MUTE);
	
	printf("\nEnter your choice: ");
}

void get_usb_power_status()
{
	mrm_err result;
	char status;

	if( (result = sdp_usb_get_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_usb_get_status fail. error code=0x%08x\n", result);
		return;
	}

	printf("Front-end USB Power is %s\n", status ? "ON" : "OFF");
}

void set_usb_power_status()
{
	mrm_err result;
	int s;
		
	printf("Enable 1 ? Disable 0:");
	scanf("%d", &s);
	
	WAIT_ENTER();
	
	if( s < 0 || s >1)
	{
		printf("Error range\n");
		return;
	}

	if( s )
	{
		if( (result = sdp_usb_enable()) != MRM_ERR_NO_ERROR )
		{
			printf("sdp_usb_auto_detect_enable fail. error code=0x%08x\n", result);
			return;
		}	
	}
	else
	{
		if( (result = sdp_usb_disable()) != MRM_ERR_NO_ERROR )
		{
			printf("sdp_usb_auto_detect_disable fail. error code=0x%08x\n", result);
			return;
		}
	}

	printf("Success\n");
}

void get_speaker_status()
{
	mrm_err result;
	char status;

	if( (result = sdp_speaker_get_status(&status)) != MRM_ERR_NO_ERROR )
	{
		printf("sdp_speaker_get_status fail. error code=0x%08x\n", result);
		return;
	}

	printf("The speaker is %s\n", status ? "ON" : "OFF");
}

void set_speaker_status()
{
	mrm_err result;
	int s;
		
	printf("Enable 1 ? Disable 0:");
	scanf("%d", &s);
	
	WAIT_ENTER();
	
	if( s < 0 || s >1)
	{
		printf("Error range\n");
		return;
	}

	if( s )
	{
		if( (result = sdp_speaker_enable()) != MRM_ERR_NO_ERROR )
		{
			printf("sdp_speaker_enable fail. error code=0x%08x\n", result);
			return;
		}	
	}
	else
	{
		if( (result = sdp_speaker_disable()) != MRM_ERR_NO_ERROR )
		{
			printf("sdp_speaker_disable fail. error code=0x%08x\n", result);
			return;
		}
	}

	printf("Success\n");
}

void peripheral_main()
{
	int op;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		peripheral_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;
				
		switch(op)
		{
			case GET_FRONT_END_USB_POWER_STATUS:
				get_usb_power_status();
				break;
			case SET_FRONT_END_USB_POWER:
				set_usb_power_status();
				break;
			case GET_SPEAKER_MUTE_STATUS:
				get_speaker_status();
				break;
			case SET_SPEAKER_MUTE:
				set_speaker_status();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
