#include <stdio.h>

#include "common.h"
#include "vcil/library.h"
#include "vcil/firmware.h"

enum firmware_function{
	FIRMWARE_GET_VERSION = 1,
	FIRMWARE_RESET
};

static void show_welcome_title(void)
{
	printf("VCIL SDK Sample -- library version:%s\n", library_version);
	printf(">> Firmware Menu\n\n");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) Get firmware version\n", FIRMWARE_GET_VERSION);
	printf("%d) Reset firmware\n", FIRMWARE_RESET);
		
	printf("\nEnter your choice: ");
}

void get_firmware_version()
{
	mrm_err result;
	char fw_version[VCIL_MAXIMUM_FIRMWARE_VERSION_LENGTH];

	if( (result = vcil_firmware_get_version(fw_version)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_firmware_get_version fail. error code=0x%08x\n", result);
		return;
	}

	printf("Firmware version is: %s\n", fw_version);
}

void reset_firmware()
{
	mrm_err result;

	if( (result = vcil_firmware_reset()) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_firmware_reset fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void firmware_main()
{
	int op;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;

		switch(op)
		{
			case FIRMWARE_GET_VERSION:
				get_firmware_version();
				break;
			case FIRMWARE_RESET:
				reset_firmware();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
