#include <stdio.h>

#include "common.h"
#include "vcil/library.h"

enum firmware_function{
	MODULE_GET_MODE = 1,
	MODULE_SET_MODE
};

static void show_welcome_title(void)
{
	printf("VCIL SDK Sample -- library version:%s\n", library_version);
	printf(">> Module Control Menu\n\n");
}

static void page_menu()
{
	printf("0) back\n");

	printf("%d) Get Mode\n", MODULE_GET_MODE);
	printf("%d) Set Mode\n", MODULE_SET_MODE);
		
	printf("\nEnter your choice: ");
}

void get_module_mode()
{
	mrm_err result;

	vcil_mode can_port0, can_port1, j1708_port0;

	if( (result = vcil_get_mode(&can_port0, &can_port1, &j1708_port0)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_get_mode fail. error code=0x%08x\n", result);
		return;
	}

	printf("CAN Port 0 is:%s\n", can_port0 == VCIL_MODE_CAN ? "CAN" : (can_port0 == VCIL_MODE_J1939 ? "J1939":"OBD2") );
	printf("CAN Port 1 is:%s\n", can_port1 == VCIL_MODE_CAN ? "CAN" : (can_port1 == VCIL_MODE_J1939 ? "J1939":"OBD2") );
	printf("J1708 Port 0 is:%s\n", j1708_port0 == VCIL_MODE_J1708 ? "J1708" : "J1587");
}

void set_module_mode()
{
	mrm_err result;
	vcil_mode can_port0, can_port1, j1708_port0;
	
	printf("CAN Port 0:\n");
	printf("0) CAN\n");
	printf("1) J1939\n");
	printf("2) OBD2\n");
	unsigned int p0;
	scanf("%u", &p0);
	
	WAIT_ENTER();
	
	if( p0 > 2)
	{
		printf("Error range\n");
		return;
	}
	
	if(p0 == 0)
		can_port0 = VCIL_MODE_CAN;
	else if(p0 == 1)
		can_port0 = VCIL_MODE_J1939;
	else
		can_port0 = VCIL_MODE_OBD2;
		
	printf("CAN Port 1:\n");
	printf("0) CAN\n");
	printf("1) J1939\n");
	printf("2) OBD2\n");
	unsigned int p1;
	scanf("%u", &p1);
	
	WAIT_ENTER();
	
	if( p1 > 2)
	{
		printf("Error range\n");
		return;
	}
	
	if(p1 == 0)
		can_port1 = VCIL_MODE_CAN;
	else if(p1 == 1)
		can_port1 = VCIL_MODE_J1939;
	else
		can_port1 = VCIL_MODE_OBD2;
		
	printf("J1708 Port 0:\n");
	printf("0) J1708\n");
	printf("1) J1587\n");
	unsigned int j0;
	scanf("%u", &j0);
	
	WAIT_ENTER();
	
	if( j0 > 1)
	{
		printf("Error range\n");
		return;
	}
	
	if(j0 == 0)
		j1708_port0 = VCIL_MODE_J1708;
	else
		j1708_port0 = VCIL_MODE_J1587;
	

	if( (result = vcil_set_mode(can_port0,can_port1,j1708_port0)) != MRM_ERR_NO_ERROR )
	{
		printf("vcil_set_mode fail. error code=0x%08x\n", result);
		return;
	}

	printf("Success\n");
}

void module_control_main()
{
	int op;

	while(true)
	{
		CLEAR_SCREEN();
		show_welcome_title();
		page_menu();

		if( SCANF("%d", &op) <=0 )
			op = -1;

		WAIT_ENTER();

		if(op == 0)
			break;

		switch(op)
		{
			case MODULE_GET_MODE:
				get_module_mode();
				break;
			case MODULE_SET_MODE:
				set_module_mode();
				break;
			default:
				printf("Unknown choice!\n");
				break;
		}

		printf("\nPress ENTER to continue. ");
		WAIT_ENTER();

	}
}
